import requests
from requests.auth import HTTPDigestAuth, HTTPBasicAuth
import os


class HTTPMessage:

    def __init__(self, body=""):
        self.__body = body
        self.__headers = {}
        self.__code = 200
        self.__auth__ = None
        self.__attach = None

    @property
    def body(self):
        return self.__body

    @body.setter
    def body(self, text):
        self.__body = text

    @property
    def code(self):
        return self.__code

    @code.setter
    def code(self, code):
        self.__code = code

    @property
    def headers(self):
        return self.__headers

    @headers.setter
    def headers(self, headers):
        self.__headers = headers

    def set_body_from_file(self, path):
        with open(path, 'r') as file:
            self.__body = file.read()

    @property
    def auth(self):
        class auth:
            @staticmethod
            def digest(login, password):
                self.__auth__ = HTTPDigestAuth(login, password)

            @staticmethod
            def basic(login, password):
                self.__auth__ = HTTPBasicAuth(login, password)

        return auth

    def attach(self, *files):
        self.__attach = {}
        for file in files:
            if type(file) is list:
                self.__attach[file[0]] = open(file[1], 'rb')
            else:
                self.__attach[os.path.basename(file)] = open(file, 'rb')

    def post(self, url, port, metod="/", timeout=10):
        endpoint = f"{url}:{port}{metod}"
        response = requests.post(endpoint,
                                 headers=self.__headers,
                                 data=self.__body.encode(),
                                 timeout=timeout,
                                 verify=False,
                                 auth=self.__auth__,
                                 files=self.__attach)
        return self.__make_response_message(response)

    def put(self, url, port, metod="/", timeout=10):
        endpoint = f"{url}:{port}{metod}"
        response = requests.put(endpoint,
                                headers=self.__headers,
                                data=self.__body.encode(),
                                timeout=timeout,
                                verify=False,
                                auth=self.__auth__,
                                files=self.__attach)
        return self.__make_response_message(response)

    def patch(self, url, port, metod="/", timeout=10):
        endpoint = f"{url}:{port}{metod}"
        response = requests.patch(endpoint,
                                  headers=self.__headers,
                                  data=self.__body.encode(),
                                  timeout=timeout,
                                  verify=False,
                                  auth=self.__auth__,
                                  files=self.__attach)
        return self.__make_response_message(response)

    def delete(self, url, port, metod="/", timeout=10):
        endpoint = f"{url}:{port}{metod}"
        response = requests.delete(endpoint,
                                   headers=self.__headers,
                                   data=self.__body.encode(),
                                   timeout=timeout,
                                   verify=False,
                                   auth=self.__auth__,
                                   files=self.__attach)
        return self.__make_response_message(response)

    def get(self, url, port, metod="/", timeout=10):
        endpoint = f"{url}:{port}{metod}"
        response = requests.get(endpoint,
                                headers=self.__headers,
                                timeout=timeout,
                                verify=False,
                                auth=self.__auth__)
        return self.__make_response_message(response)

    @staticmethod
    def __make_response_message(response):
        response_message = HTTPMessage(response.text)
        response_message.headers = response.headers
        response_message.__code = response.status_code
        return response_message



from psycopg2.extras import RealDictCursor
from contextlib import closing
import psycopg2.errors
import psycopg2
import json


class Errors:
    unprocessable_entity = 0
    invalid_request = -1
    database_not_avalable = -2


class Postgre:

    def __init__(self, host, port, database, user, password):
        self.__host = host
        self.__port = port
        self.__database = database
        self.__user = user
        self.__password = password

    def execute(self, *sql, array=True):
        try:
            with closing(psycopg2.connect(dbname=self.__database,
                                          user=self.__user,
                                          password=self.__password,
                                          host=self.__host,
                                          port=self.__port)) as connect:
                with connect.cursor(cursor_factory=RealDictCursor) as cur:
                    try:
                        cur.execute(*sql)
                    except Exception:
                        return -1
                    if cur.description:
                        result = cur.fetchall() if array else cur.fetchone()
                        return json.dumps(result, indent=4, default=str, ensure_ascii=False)
                    else:
                        connect.commit()
                        return cur.rowcount
        except psycopg2.OperationalError:
            return -2



import json


class TemplateJSON:

    def __init__(self, body="{}"):
        self._dictionary = json.loads(body)

    def __getitem__(self, name):
        return self._dictionary[name]

    def __setitem__(self, key, value):
        self._dictionary[key] = value

    def __delitem__(self, key):
        self._dictionary.pop(key)

    def set_body_from_file(self, path):
        with open(path, 'r') as file:
            self._dictionary = json.loads(file.read())

    def __get_single_value_json(self, name):
        return self._dictionary[name]

    @property
    def body(self):
        return json.dumps(self._dictionary)

    @body.setter
    def body(self, body):
        self._dictionary = json.loads(body)

    @property
    def json(self):
        return self._dictionary



import cx_Oracle
import json


class Errors:
    unprocessable_entity = 0
    invalid_request = -1
    database_not_avalable = -2


class Oracle:

    def __init__(self, host, port, sid, user, password, lib_dir=None, encoding='UTF-8'):
        self.__host = host
        self.__port = port
        self.__sid = sid
        self.__user = user
        self.__password = password
        self.__lib_dir = lib_dir
        self.__encoding = encoding

    def execute(self, *sql, array=True):
        try:
            if self.__lib_dir:
                cx_Oracle.init_oracle_client(lib_dir=self.__lib_dir)
                # https://www.oracle.com/database/technologies/instant-client.html
            with cx_Oracle.connect(
                    user=self.__user,
                    password=self.__password,
                    dsn=cx_Oracle.makedsn(host=self.__host, port=self.__port, sid=self.__sid),
                    encoding=self.__encoding) as connect:
                with connect.cursor() as cur:
                    try:
                        cur.execute(*sql)
                    except cx_Oracle.Error:
                        return -1
                    if cur.description:
                        cur.rowfactory = lambda *args: dict(zip([d[0] for d in cur.description], args))
                        result = cur.fetchall() if array else cur.fetchone()
                        return json.dumps(result, indent=4, default=str, ensure_ascii=False)
                    else:
                        connect.commit()
                        return cur.rowcount
        except cx_Oracle.Error:
            return -2
